@@ -141,6 +141,10 @@ void help_display(Screen *t)
 						case 'B':
 						case 'f':
 						case 'F':
+						case 's':
+						case 'S':
+						case 'z':
+						case 'Z':
 							++str;
 							break;
 						case '|':
@@ -232,6 +236,18 @@ void help_display(Screen *t)
 							++str;
 							--x;
 							continue;
+						case 'z':
+						case 'Z':
+							atr ^= DOUBLE_UNDERLINE;
+							++str;
+							--x;
+							continue;
+						case 's':
+						case 'S':
+							atr ^= CROSSED_OUT;
+							++str;
+							--x;
+							continue;
 						case 0:	
 							--x;
 							continue;
