@@ -145,11 +145,13 @@ typedef RETSIGTYPE (*sighandler_t)(int);
 #define REINSTALL_SIGHANDLER(sig, handler) do {} while(0)
 #endif
 
-/* wrapper to hide signal interface differrencies */
+/* wrapper to hide signal interface differences */
 int joe_set_signal(int signum, sighandler_t handler);
 
 /* Simple parsers */
 int parse_ws(const char * *p,int cmt);
+int parse_wsn(const char * *p,int cmt);
+int parse_wsl(const char * *p,int cmt);
 int parse_ident(const char **p,char *buf,ptrdiff_t len);
 int parse_kw(const char **p,const char *kw);
 int parse_tows(const char **p,char *buf);
