@@ -754,7 +754,7 @@ static void mdisp(SCRN *t, ptrdiff_t y, const char *s)
 		ofst = 0;
 	else
 		ofst = len - (t->co);
-	genfmt(t, 0, y, ofst, s, BG_COLOR(bg_msg), 1);
+	genfmt(t, 0, y, ofst, s, BG_COLOR(bg_msg), 0, 1);
 	t->updtab[y] = 1;
 }
 
